
install_packages()
{
	if [[ $InstallPackages == yes ]]; then
		display_alert "Installing Required packages" "" ""
		for Package_Name in $Install_Package_List; do
			display_alert "Installing $Package_Name" "" ""
			apt-get install -y -q $Package_Name --install-recommends
			#TODO - Error checking
		done
	fi
}

remove_packages()
{ 
	display_alert "Removing unwanted and temporary Packages" "" "info"
	apt purge -y --allow-change-held-packages linux-headers-next-eureka
	rm -rf /usr/src
}

#--------------------------------------------------------------------------------------------------------------------------------
# Let's have unique way of displaying alerts
#--------------------------------------------------------------------------------------------------------------------------------
display_alert()
{
	# log function parameters to install.log
	[[ -n $DEST ]] && echo "Displaying message: $@" >> $DEST/debug/output.log

	local tmp=""
	[[ -n $2 ]] && tmp="[\e[0;33m $2 \x1B[0m]"

	case $3 in
		err)
		echo -e "[\e[0;31m error \x1B[0m] $1 $tmp"
		;;

		wrn)
		echo -e "[\e[0;35m warn \x1B[0m] $1 $tmp"
		;;

		ext)
		echo -e "[\e[0;32m o.k. \x1B[0m] \e[1;32m$1\x1B[0m $tmp"
		;;

		info)
		echo -e "[\e[0;32m o.k. \x1B[0m] $1 $tmp"
		;;

		dbg)
		echo -e "[\e[0;34m DEBUG \x1B[0m] $1 $tmp"
		;;

		*)
		echo -e "[\e[0;32m .... \x1B[0m] $1 $tmp"
		;;
	esac
}

setup_wiegand_driver()
{
	local KV=$1
	display_alert "Setting up Wiegand Driver" "$KV" ""
	cd /root
	cp -r /tmp/overlay/sources/wiegand_driver wiegand_driver
	cd /root/wiegand_driver/wiegandOUT
	make
	cp wiegandOUT.ko /lib/modules/$KV-eureka/kernel/drivers/misc
	cd /root
	rm -rf wiegand_driver
	md5sum /lib/modules/$KV-eureka/kernel/drivers/misc/wiegandOUT.ko > /var/lib/dpkg/info/wiegandOUT.md5sums
}

instal_rvi_dependencies()
{
	se_ver=v7.0.33922
	display_alert "Installing RVI dependencies" "" ""
	cp -r /tmp/overlay/etc/ld.so.conf.d/hid.conf /etc/ld.so.conf.d/
	tar -xf /tmp/overlay/sources/debs/libturbjpeg.tar.gz -C /
	tar -xf /tmp/overlay/sources/debs/paho-mqtt-cpp.tar.gz -C /
	tar -xf /tmp/overlay/sources/Lumidigm/V4/SEngine_${se_ver}-arm64.tar.gz -C /root
	cp /tmp/overlay/sources/Lumidigm/V4/usr/local/lib/libVCOMExample.so /usr/local/lib
	ln -s /usr/local/lib/libVCOMExample.so /usr/local/lib/libVCOMExample.so.1
	cd root/SEngine_${se_ver}-arm64
	awk '!/read/' install.sh > temp && mv temp install.sh
	chmod +x install.sh
	sudo ./install.sh
	ldconfig
	cd ..
	rm -rf SEngine_${se_ver}-arm64
}

setup_qc_test()
{
	display_alert "Setting up RB25F Test Application" "" ""
	cd /root
	cp -r /tmp/overlay/sources/EurekaTest/Release/qctest /usr/local/bin
}

install_all_debs()
{ 
	local debfiles=()

	local debdir="/tmp/overlay/sources/debs"

	for deb in ${debdir%%:*}/*.deb; do
		debfiles+=($(basename $deb))
	done
	for debfile in ${debfiles[@]}; do
		display_alert "Installing $debfile" "" ""
		dpkg -i $debdir/$debfile
	done
}

download_additional_debs()
{ 
	display_alert "Preparing First run apps for install" "" ""
	mkdir /root/tmpfiles
	cd /root/tmpfiles
	cp /tmp/overlay/sources/debs/rvi-*-arm64.deb .
	apt-get install -y libavahi-common3 libavahi-common-data libavahi-core7 libev4 libuv1 libwebsockets8 libnss-mdns
	cp /tmp/overlay/sources/zerotier-one_1.2.12_arm64.deb .
	apt-get download avahi-daemon
	mkdir mosquitto
	cd mosquitto
	apt-get download mosquitto
	display_alert "Ignore Permission denied warnings, (Known bug in apt)" "" "info"
}

install_fixup_script()
{ 
	display_alert "Installing Custom U-Boot Env script" "" ""
	cd /boot
	mkimage -T script -C none 'fixup.scr' -d /tmp/overlay/boot/fixup.cmd fixup.scr
}