$(document).ready(readyFn);

let captureEnabled = true;
let minutiaEnabled = true;
let minCount = 0;
let statusSocket;

function setMinCount() {
  $('#minCount').text(`Minutia Count: ${minCount}`);
}

function setUserInstruction(text) {
  $('#userInstruction').text(text);
}

function clearScreen(width, height) {
  let canvas = document.getElementById('fingerCanvas');
  let ctx = canvas.getContext('2d');
  ctx.beginPath();
  ctx.rect(0, 0, width, height);
  ctx.fillStyle = '#ffffff';
  ctx.fill();

  canvas = document.getElementById('minutiaeCanvas');
  ctx = canvas.getContext('2d');
  ctx.beginPath();
  ctx.rect(0, 0, width, height);
  ctx.fillStyle = '#ffffff';
  ctx.fill();
}

function setCaptureViewable(state) {
  const canvas = document.getElementById('fingerCanvas');
  if (state === false) {
    canvas.style.display = 'none';
  } else {
    canvas.style.display = '';
  }
}

function setMinViewable(state) {
  const canvas = document.getElementById('minutiaeCanvas');
  if (state === false) {
    canvas.style.display = 'none';
  } else {
    canvas.style.display = '';
  }
}

function toggleView(btn) {
  const { data: btnType } = btn;
  if (btnType === 'fingerCanvas') {
    captureEnabled = !captureEnabled;
    setCaptureViewable(captureEnabled);
  } else if (btnType === 'minutiaeCanvas') {
    minutiaEnabled = !minutiaEnabled;
    setMinViewable(minutiaEnabled);
  }
}

function readyFn() {
  $('#toggleMinutiae').prop('disabled', true);
  $('#toggleCapture').prop('disabled', true);
  $('#getTemplate').click(getTemplate);
  $('#toggleCapture').click('fingerCanvas', toggleView);
  $('#toggleMinutiae').click('minutiaeCanvas', toggleView);
  setMinCount();

  statusSocket = new WebSocket('ws://10.190.6.84:3000/m2m');
  statusSocket.onopen = (event) => {
    statusSocket.send(JSON.stringify({ id: 'jonweb', type: 'WEB_CLIENT' }));
  };

  statusSocket.onmessage = function (rxMsg) {
    const msg = JSON.parse(rxMsg.data);
    const { event } = msg;
    const { FINGER } = event;
    if ('CAPTURING' === FINGER) {
      setUserInstruction("Place finger on sensor.");
    } else if ('LIFT' === FINGER) {
      setUserInstruction("Lift finger off sensor.");
    } else if ('CAPTURED' === FIGER) {
      setUserInstruction("Captured Finger!");
    }
  };
}

function getTemplate() {
  console.log('getTemplate.');
  $('#getTemplate').prop('disabled', true);
  minCount = 0;
  setMinViewable(true);
  setCaptureViewable(true);

  $.ajax({
    url: 'http://10.190.6.84:3000/2/finger/template',
    success: templateData,
    dataType: 'json',
  });
}

function drawImageFile(imgData, width, height, type, coords) {
  const canvas = document.getElementById('fingerCanvas');
  canvas.wdith = width;
  canvas.height = height;

  canvas.style.width = `${width}px`;
  canvas.style.height = `${height}px`;

  const ctx = canvas.getContext('2d');

  // var uInt8Array = new Uint8Array(imgData);
  const uInt8Array = imgData;
  let i = uInt8Array.length;
  const binaryString = [i];
  while (i--) {
    binaryString[i] = String.fromCharCode(uInt8Array[i]);
  }
  const data = binaryString.join('');

  const base64 = window.btoa(data);

  const img = new Image();
  img.src = `data:${type};base64,${base64}`;
  img.onload = function () {
    ctx.drawImage(img, coords[0], coords[1], canvas.width, canvas.height);
  };
  img.onerror = function (err) {
    console.log('Img Onerror:', err);
  };
}

async function renderLine(width, height) {
  return new Promise((resolve, reject) => {
    const canvas = document.getElementById('renderLine');
    width += 50;
    height += 50;
    canvas.wdith = width;
    canvas.height = height;

    canvas.style.width = `${width}px`;
    canvas.style.height = `${height}px`;

    const ctx = canvas.getContext('2d');

    let startY = 0;
    const endY = height;

    ctx.beginPath();
    ctx.rect(0, 0, width, height);
    ctx.fillStyle = '#ffffff';
    ctx.fill();

    setCaptureViewable(true);

    const interval = setInterval(() => {
      ctx.clearRect(0, 0, width, height);
      ctx.beginPath();
      ctx.rect(0, startY, width, height);
      ctx.fillStyle = '#ffffff';
      ctx.fill();

      startY += 2;

      if (startY >= endY) {
        clearInterval(interval);
        resolve();
      }
    }, 15);
  });
}

function drawMinutiaPoint(canvas, x, y, type, rotation, index) {
  setTimeout(() => {
    canvas.save();
    canvas.lineWidth = 1;
    canvas.strokeStyle = '#FF0000';
    canvas.beginPath();
    // canvas.moveTo(x, y);
    // canvas.lineTo(x + 2, y + 2);
    canvas.arc(x, y, 6, 0, 2 * Math.PI);
    canvas.stroke();

    canvas.beginPath();
    canvas.strokeStyle = '#0015ff';
    canvas.moveTo(x, y - 5);
    canvas.lineTo(x, y + 5);
    canvas.stroke();

    canvas.beginPath();
    canvas.strokeStyle = '#0015ff';
    canvas.moveTo(x - 5, y);
    canvas.lineTo(x + 5, y);
    canvas.stroke();

    canvas.restore();
    minCount++;
    setMinCount();
  }, 1);
}

function drawMinutia(minPoints, width, height) {
  const canvas = document.getElementById('minutiaeCanvas');
  canvas.wdith = width;
  canvas.height = height;

  canvas.style.width = `${width}px`;
  canvas.style.height = `${height}px`;

  const ctx = canvas.getContext('2d');
  ctx.save();

  minPoints.forEach((singlePoint, index) => {
    const {
      x, y, type, rotation,
    } = singlePoint;
    drawMinutiaPoint(ctx, x, y, type, rotation, index);
  });
}

function templateData(captureData, status) {
  $('#getTemplate').prop('disabled', false);
  $('#toggleMinutiae').prop('disabled', false);
  $('#toggleCapture').prop('disabled', false);
  if (status === 'success') {
    const { capture, template, minutia } = captureData;

    const {
      width, height, data, type, quality,
    } = capture;

    //setCaptureViewable(false);

    drawImageFile(data, width, height, type, [0, 0]);
    //renderLine(width, height)
    drawMinutia(minutia, width, height);
  }
}

