#!/bin/bash

ramDiskSizeBytes=460800
rdMountPoint="/crypted1"
cmDevice="/dev/mapper/vcom"
hbserverPath="/usr/local/eureka/hbserver"
hbDatabaseName="primary.sqlite"

exitCodeCheck() {
if [ $? -eq 0 ]
then
  echo "$1"
else
  echo "$2" >&2
fi
}

createEncryptedDatabase() {
cat <<EOT > /tmp/createSQLDB.sql
PRAGMA rekey=$1;
EOT

rm $hbserverPath/$hbDatabaseName
exitCodeCheck "Deleted primary.sqlite" "Failed to delete primary.sqlite"
sqleet $hbserverPath/$hbDatabaseName < /tmp/createSQLDB.sql
exitCodeCheck "Created encrypted SQLite database." "Failed to create SQLite encrypted database."

dbCheck=$(sqleet mydb.sqlite "pragma key=$1;" ".dump" ".exit" | grep ok | wc -l)
if [ "$dbCheck" == 1 ]; then
        echo "SQLite db encrypted successfully";
        true
    else
        echo "SQLite db encryption failed.";
        false
    fi
}

cMisMounted() {
    isMounted=$(cryptmount --status | grep vcom | grep " mounted" | wc -l)
    if [ "$isMounted" == 1 ]; then
        echo "Crypto FS loaded.";
        true
    else
        echo "Crypto failed to load.";
        false
    fi
}

isRamDiskModuleLoaded () {
    isModuleLoaded=$(lsmod | grep brd | wc -l)
    if [ "$isModuleLoaded" == 1 ]; then
        echo "RAM disk module loaded.";
        true
    else
        echo "RAM disk module not loaded.";
        false
    fi
}

loadRamDiskModule () {
    modprobe brd rd_size=$ramDiskSizeBytes
}

createCmtab() {
cat <<EOT > /etc/cryptmount/cmtab
vcom {
    dev=/dev/ram0 dir=$rdMountPoint
    fstype=ext4 mountoptions=defaults cipher=aes
    keyfile=/tmp/crypted1.key flags=nofsck
    keyformat=builtin
}
EOT
}

cmKeyGenerationScript() {
cat << EOT > /tmp/generate_key.sh
#!/usr/bin/expect -f

set timeout -1
spawn /usr/bin/cryptmount --generate-key 32 vcom
expect "Enter new password for target"
send -- "$1\r"
expect "Confirm password"
send -- "$1\r"
expect eof
EOT

chmod +x /tmp/generate_key.sh
}

cmPrepareScript() {
cat << EOT > /tmp/prepare_cm.sh
#!/usr/bin/expect -f

set timeout -1
spawn /usr/bin/cryptmount --prepare vcom
expect "Enter password for target"
send -- "$1\r"
expect eof
EOT

chmod +x /tmp/prepare_cm.sh
}

cmMount() {
cryptmount --passwd-fd 3 vcom  3<  <(echo "$1")
}

##################
# start here
##################

if [ "$1" == "" ]; then
        echo "Argument 1 must be an encryption key.";
        exit 1
    fi

service rvi stop
cryptmount -u vcom
umount /crypted1
rmmod brd

if isRamDiskModuleLoaded; then
    # module loaded
    echo "RAM disk module OK"
else
    # module not loaded
    loadRamDiskModule;
    # check it did load.
    if isRamDiskModuleLoaded; then
        echo "Module loaded successfully."
    else
        echo "Failed to load module."
        exit -1;
    fi
fi


rm -rf /crypted1
rm -f /tmp/crypted1.key

# make the cryptmount config file.
createCmtab

# create the key generation script.
cmKeyGenerationScript $1

# prepare script
cmPrepareScript $1

# generate the key file
/usr/bin/expect /tmp/generate_key.sh
# setup the cm device
#sleep 5
/usr/bin/expect /tmp/prepare_cm.sh
#sleep 5

mke2fs $cmDevice
cryptmount --release vcom
mkdir $rdMountPoint
#cryptmount --passwd-fd 3 vcom  3<  <(echo "$1")
cmMount $1

if cMisMounted; then
# Not using -p to make sure that the initial crypto fs exists already. Otherwise it may force it and the error is hidden.
mkdir $rdMountPoint/IDDB

cd /usr/local/HID_Global/Lumidigm/var/lib
rm -r IDDB
ln -s $rdMountPoint/IDDB IDDB
#createEncryptedDatabase $1
# Recreates the finger DB.
service rvi restart
else
echo "---------------------------------------" >&2
echo "FAILED TO STARTED ENCRYPTED CONTAINER." >&2
echo "FAILED TO STARTED ENCRYPTED CONTAINER." >&2
echo "FAILED TO STARTED ENCRYPTED CONTAINER." >&2
echo "---------------------------------------" >&2
fi
